/*
	trace de doite (gauche a droite) : algorithme du point milieu (bresenham)
	
	mailto:jciehl@bat710.univ-lyon1.fr
 */

#include <GL/glut.h>

#include "draw.h"

// trace une droite de x1, y1 vers x2, y2
// direction principale +x, +y
void draw_line_px(int x1, int y1, int x2, int y2)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy+dy - dx;
	int incx= dy+dy;
	int incy= (dy - dx)+(dy - dx);
	
	int x= x1;
	int y= y1;
	
	draw_pixel(x, y);
	while(x < x2)
	{
		if(d<=0)
		{
			// M est au dessus de la droite
			d+= incx;
			x++;
		}
		else
		{	// M est au dessous de la droite
			d+= incy;
			x++;
			y++;
		}
		
		draw_pixel(x, y);
	}
}

// direction princiaple +y, +x
void draw_line_py(int x1, int y1, int x2, int y2)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy - dx-dx;
	int incx= (dy - dx)+(dy - dx);
	int incy= -dx-dx;
	
	int x= x1;
	int y= y1;
	
	draw_pixel(x, y);
	while(y < y2)
	{
		if(d>=0)
		{
			d+= incy;
			y++;
		}
		else
		{
			d+= incx;
			x++;
			y++;
		}
		
		draw_pixel(x, y);
	}
}

// direction principale +x, -y
void draw_line_mx(int x1, int y1, int x2, int y2)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy+dy + dx;
	int incx= dy+dy;
	int incy= (dy+dx) + (dy+dx);
	
	int x= x1;
	int y= y1;
	
	draw_pixel(x, y);
	while(x < x2)
	{
		if(d>=0)
		{
			d+= incx;
			x++;
		}
		else
		{
			d+= incy;
			x++;
			y--;
		}
		
		draw_pixel(x, y);
	}
}

// direction principale -y, +x
void draw_line_my(int x1, int y1, int x2, int y2)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy + dx+dx;
	int incx= (dy + dx)+(dy + dx);
	int incy= dx+dx;
	
	int x= x1;
	int y= y1;
	
	draw_pixel(x, y);
	while(y > y2)
	{
		if(d<=0)
		{
			d+= incy;
			y--;
		}
		else
		{
			d+= incx;
			x++;
			y--;
		}
		
		draw_pixel(x, y);
	}
}


void draw_line(int x1, int y1, int x2, int y2)
{
	int dx;
	int dy;
	int t;

	if(x1 > x2)
	{
		// permuter les extremites == trace de gauche a droite
		t= x1; x1= x2; x2= t;
		t= y1; y1= y2; y2= t;
	}
	
	// dx >= 0
	dx= x2 - x1;
	dy= y2 - y1;

	// determiner la direction principale	
	if(dy >= 0)
	{
		if(dx > dy)
			draw_line_px(x1, y1, x2, y2);
		else
			draw_line_py(x1, y1, x2, y2);
	}
	else
	{
		if(dx > -dy)
			draw_line_mx(x1, y1, x2, y2);
		else
			draw_line_my(x1, y1, x2, y2);
	}
}

void main_droite(void)
{
    glColor3f(1.0, 1.0, 0.0);
	draw_line(10, 10, 30, 18);

	glColor3f(1.0, 0.0, 0.0);
	draw_line(10, 18, 30, 10);

	glColor3f(0.0, 1.0, 0.0);	
	draw_line(10, 10, 18, 30);
	
	glColor3f(1.0, 0.0, 1.0);
	draw_line(10, 30, 18, 10);
}
