/*
	trace de triangle

	a faire :
		. interpolation bilineaire des attributs des sommets (couleur, transparence, profondeur, normale, coordonnees de textures)
		. clipping
		. triangle_strip (reutilisation des scan deja calcules)
	
	mailto:jciehl@bat710.univ-lyon1.fr
*/


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include <GL/glut.h>

#include "draw.h"
#include "droite.h"

// trace une droite de x1, y1 vers x2, y2
// direction principale +x, +y
void draw_edge_px(int x1, int y1, int x2, int y2, int *scan)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy+dy - dx;
	int incx= dy+dy;
	int incy= (dy - dx)+(dy - dx);
	
	int x= x1;
	int y= y1;
	
	scan[y]= x;
	while(x < x2)
	{
		if(d<=0)
		{
			d+= incx;
			x++;
		}
		else
		{
			d+= incy;
			x++;
			y++;
		}
		
		scan[y]= x;
	}
}

// direction principale +y, +x
void draw_edge_py(int x1, int y1, int x2, int y2, int *scan)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy - dx-dx;
	int incx= (dy - dx)+(dy - dx);
	int incy= -dx-dx;
	
	int x= x1;
	int y= y1;
	
	scan[y]= x;
	while(y < y2)
	{
		if(d>=0)
		{
			d+= incy;
			y++;
		}
		else
		{
			d+= incx;
			x++;
			y++;
		}
		
		scan[y]= x;
	}
}

// direction principale +x, -y
void draw_edge_mx(int x1, int y1, int x2, int y2, int *scan)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy+dy + dx;
	int incx= dy+dy;
	int incy= (dy+dx) + (dy+dx);
	
	int x= x1;
	int y= y1;
	
	scan[y]= x;
	while(x < x2)
	{
		if(d>=0)
		{
			d+= incx;
			x++;
		}
		else
		{
			d+= incy;
			x++;
			y--;
		}
		
		scan[y]= x;
	}
}

// direction principale -y, +x
void draw_edge_my(int x1, int y1, int x2, int y2, int *scan)
{
	int dx= x2 - x1;
	int dy= y2 - y1;
	int d= dy + dx+dx;
	int incx= (dy + dx)+(dy + dx);
	int incy= dx+dx;
	
	int x= x1;
	int y= y1;
	
	scan[y]= x;
	while(y > y2)
	{
		if(d<=0)
		{
			d+= incy;
			y--;
		}
		else
		{
			d+= incx;
			x++;
			y--;
		}
		
		scan[y]= x;
	}
}


void draw_edge(int x1, int y1, int x2, int y2, int *scan)
{
	int dx;
	int dy;
	int t;

	if(x1 > x2)
	{
		// permuter les extremites == trace de gauche a droite
		t= x1; x1= x2; x2= t;
		t= y1; y1= y2; y2= t;
	}
	
	// dx >= 0
	dx= x2 - x1;
	dy= y2 - y1;

	// determiner la direction principale	
	if(dy >= 0)
	{
		if(dx > dy)
			draw_edge_px(x1, y1, x2, y2, scan);
		else
			draw_edge_py(x1, y1, x2, y2, scan);
	}
	else
	{
		if(dx > -dy)
			draw_edge_mx(x1, y1, x2, y2, scan);
		else
			draw_edge_my(x1, y1, x2, y2, scan);
	}
}


void draw_triangle(int x[3], int y[3], int hauteur)
{
	int ordre[3];
	int *minx, *maxx;
	int i, j, miny;
	
	// allouer les extremites des segments pour chaque ligne
	minx= (int *) calloc(hauteur, sizeof(int));
	assert(minx);

	maxx= (int *) calloc(hauteur, sizeof(int));
	assert(maxx);
	
	// determiner l'ordre de trace
	// .  etape 1: trouver le sommet superieur
	ordre[0]= 0;
	for(i= 1; i < 3; i++)
		if(y[i] > y[ordre[0]])
			ordre[0]= i;

	// .  etape 1b: ordonne les autres sommets	
	for(j= 1; j<3; j++)
		ordre[j]= (ordre[0] + j) % 3;
	
	// .  etape 2: trouver le sommet le plus a gauche et le plus a droite du sommet superieur
	if(x[ordre[1]] > x[ordre[2]])
	{
		// permutter les sommets
		i= ordre[1];
		ordre[1]= ordre[2];
		ordre[2]= i;
	}

	printf("ordre de trace %d %d, %d %d, %d %d\n", 
		x[ordre[0]], y[ordre[0]], 
		x[ordre[1]], y[ordre[1]], 
		x[ordre[2]], y[ordre[2]]);

	// determiner le segment interieur du triangle pour chaque ligne
	// .  etape 1: segment sommet superieur -> sommet gauche
	draw_edge(x[ordre[0]], y[ordre[0]], x[ordre[1]], y[ordre[1]], minx);
	
	// .  etape 2: segment sommet superieur -> sommet droit
	draw_edge(x[ordre[0]], y[ordre[0]], x[ordre[2]], y[ordre[2]], maxx);

	// .  etape 3: segment sommet gauche -> sommet droit
	if(y[ordre[1]] > y[ordre[2]])
		draw_edge(x[ordre[1]], y[ordre[1]], x[ordre[2]], y[ordre[2]], minx);
	else
		draw_edge(x[ordre[1]], y[ordre[1]], x[ordre[2]], y[ordre[2]], maxx);

	// tracer les segments de miny a y[ordre[0]]
	if(y[ordre[1]] < y[ordre[2]])
		miny= y[ordre[1]];
	else
		miny= y[ordre[2]];
	
	for(i= miny; i<=y[ordre[0]]; i++)
	{
		printf("span %d: %d %d\n", i, minx[i], maxx[i]);
	
		for(j= minx[i]; j < maxx[i]; j++)
			draw_pixel(j, i);
	}
	
	free(minx);
	free(maxx);
}

void main_triangle(void)
{
	int vx[3]= {10, 18, 30},
	    vy[3]= {10, 30, 15};

	// dessine un triangle	    
	glColor3f(0.0, 1.0, 0.0);
	draw_triangle(vx, vy, 40);
	
	// superpose les frontieres
	glColor3f(1.0, 1.0, 0.0);
	draw_line(vx[0], vy[0], vx[1], vy[1]);
	draw_line(vx[0], vy[0], vx[2], vy[2]);
	draw_line(vx[1], vy[1], vx[2], vy[2]);
}
